<?php

namespace App\Http\Controllers;

use App\domain\travelport\AddOfferInWorkbench;
use App\domain\travelport\AddTravellersInWorkbenchAction;
use App\domain\travelport\CommitReservation;
use App\domain\travelport\CreateWorkbench;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

class BookingWorkbenchController extends Controller
{
    public function createWorkbench(CreateWorkbench $createWorkbench): JsonResponse
    {
        try {
            return response()->json($createWorkbench->execute(), Response::HTTP_OK);
        } catch (Exception $e) {
            return response()->json([
                'error_message' => $e->getMessage(),
            ], Response::HTTP_BAD_REQUEST);
        }
    }

    public function addOffer(Request $request, AddOfferInWorkbench $addOfferInWorkbench): JsonResponse
    {
        try {
            $result = $addOfferInWorkbench->execute(
                $request->input('reservationId'),
                $request->input('catalogId'),
                $request->input('products')
            );

            return response()->json($result, Response::HTTP_OK);
        } catch (Exception $e) {
            return response()->json([
                'error_message' => $e->getMessage(),
            ], Response::HTTP_BAD_REQUEST);
        }
    }

    public function addTraveler(Request $request, AddTravellersInWorkbenchAction $addTravelersInWorkbenchAction): JsonResponse
    {
        try {
            $reservationId = $request->input('reservationId');
            $travellers = $request->input('traveller');

            $travellerIds = $addTravelersInWorkbenchAction->execute($reservationId, $travellers);

            return response()->json($travellerIds->toArray(), Response::HTTP_OK);
        } catch (Exception $e) {
            return response()->json([
                'error_message' => $e->getMessage(),
            ], Response::HTTP_BAD_REQUEST);
        }
    }

    public function commitReservation(Request $request, CommitReservation $commitReservation): JsonResponse
    {
        $response = $commitReservation->execute($request->input('reservationId'));

        return response()->json($response, Response::HTTP_OK);
    }

}
