<?php

namespace App\domain\travelport;

use Exception;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Exception\RequestException;
use Illuminate\Support\Facades\Log;


class FlightOfferService extends SearchOfferAPI
{
    protected array $passengerCriteria;
    protected array $searchCriteriaFlight;

    public function requestOffers(array $passengerCriteria, array $searchCriteriaFlight, ?array $searchModifiers, string $source)
    {
        try {
            $apiUrl = $this->buildApiUrl();

            $payload = $this->createPayload($passengerCriteria, $searchCriteriaFlight, $searchModifiers, $source);

//            Log::info("#payload: ", $payload);

            $response = $this->client->post($apiUrl, [
                'headers' => $this->getHeaders(),
                'json' => $payload,
                'debug' => fopen(storage_path('logs/guzzle_debug.log'), 'a'), // Specify a file to log cURL request and response
            ]);

            return json_decode($response->getBody(), true);
        } catch (Exception $e) {
            Log::error('Error requestOffers: ' . $e->getMessage());
            return null;
        } catch (GuzzleException $e) {
//            Log::error('Error Guzzle: ' . $e->getMessage());
//            return null;
//        } catch (RequestException $e) {
            // If there's an exception, you can also log the error response
            if ($e->hasResponse()) {
                $errorResponse = [
                    'statusCode' => $e->getResponse()->getStatusCode(),
                    'headers' => $e->getResponse()->getHeaders(),
                    'body' => $e->getResponse()->getBody()->getContents(),
                ];

                // Log the error response data
                Log::error('Guzzle Error Response:', $errorResponse);
            } else {
                // Log other Guzzle exceptions
                Log::error($e->getMessage());
            }
        }
    }

    protected function buildApiUrl(): string
    {
        return "https://{$this->baseURL}/{$this->apiVersion}/air/catalog/search/catalogproductofferings";
    }

    protected function createPayload(array $passengerCriteria, array $searchCriteriaFlight, ?array $searchModifiers, string $source): array
    {
        return [
            "CatalogProductOfferingsQueryRequest" => [
                "CatalogProductOfferingsRequest" => [
                    "@type" => "CatalogProductOfferingsRequestAir",
                    "offersPerPage" => 10,
                    "maxNumberOfUpsellsToReturn" => 0,
//                    "inhibitBrandContentInd" => true,
                    "contentSourceList" => [
                        $source
                    ],
                    "PassengerCriteria" => $passengerCriteria,
                    "SearchCriteriaFlight" => $searchCriteriaFlight,
                    "SearchModifiersAir" => [
                        "@type" => "SearchModifiersAir",
                        "CarrierPreference" => [
                            [
                                "@type" => "CarrierPreference",
                                "preferenceType" => "Permitted",
                                "carriers" => [
                                    "BA",
                                    "AF",
                                    "KL",
                                    "LH",
                                    "SQ"
                                ]
                            ]
                        ],
                        "CabinPreference" => $searchModifiers["cabinPreference"],
                        "ConnectionPreferences" => [
                            [
                                "@type" => "ConnectionPreferencesAir",
                                "FlightType" => [
                                    "@type" => "FlightType",
                                    "connectionType" =>
                                        $searchModifiers['flightType']
                                            ? $searchModifiers['flightType']['connectionType']
                                            : null,
                                    "excludeInterlineConnectionsInd" => true
                                ]
                            ]
                        ]
                    ]
                ]
            ]
        ];
    }
}
